# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from six import python_2_unicode_compatible
from django.conf import settings
from django.db import models
from django.utils.translation import ugettext_lazy as _

from jmb.core.db.models import DateModel, UserModel, StatusModel, OrderedModel, DefaultModel

from .managers import StoreCauseManager


@python_2_unicode_compatible
class Store(DateModel, UserModel, StatusModel, DefaultModel, OrderedModel):
    code = models.CharField(
        primary_key=True,
        max_length=15,
        verbose_name=_('code'),
    )
    name = models.CharField(
        unique=True,
        max_length=255,
        verbose_name=_('name'),
    )
    slug = models.CharField(
        unique=True,
        max_length=255,
        verbose_name=_('slug'),
    )
    description = models.TextField(
        blank=True, null=True,
        verbose_name=_('description'),
    )

    class Meta:
        verbose_name = _('store')
        verbose_name_plural = _('stores')
        ordering = ('ordering', 'name')
        permissions = (
            ("list_store", "Can list store"),
            ("detail_store", "Can detail store"),
            ("disable_store", "Can disable store"),
        )

    def __str__(self):
        return '{} {}'.format(self.code, self.name)


@python_2_unicode_compatible
class StoreDocument(DateModel, UserModel, StatusModel, OrderedModel):
    code = models.CharField(
        primary_key=True,
        max_length=15,
        verbose_name=_('code'),
    )
    name = models.CharField(
        unique=True,
        max_length=255,
        verbose_name=_('name'),
    )
    slug = models.CharField(
        unique=True,
        max_length=255,
        verbose_name=_('slug'),
    )
    description = models.TextField(
        blank=True,
        verbose_name=_('description'),
    )

    class Meta:
        verbose_name = _('store document')
        verbose_name_plural = _('store documents')
        ordering = ('ordering', 'name')
        permissions = (
            ("list_storedocument", "Can list store document"),
            ("detail_storedocument", "Can detail store document"),
            ("disable_storedocument", "Can disable store document"),
        )

    def __str__(self):
        return '{} {}'.format(self.code, self.name)


@python_2_unicode_compatible
class StoreDocumentNumber(DateModel, UserModel):
    store_document = models.ForeignKey(
        StoreDocument,
        related_name='document_numbers',
        verbose_name=_('store document'),
    )
    application = models.CharField(
        max_length=1,
        choices=settings.STORE_APP_TYPE,
        db_index=True,
    )
    year = models.PositiveIntegerField(
        db_index=True,
        verbose_name=_('year'),
    )
    last_date = models.DateField(
        db_index=True,
        verbose_name=_('last date')
    )
    last_number = models.PositiveIntegerField(
        db_index=True,
        verbose_name=_('last number'),
    )

    class Meta:
        verbose_name = _('store document number')
        verbose_name_plural = _('store document numbers')
        unique_together = ('store_document', 'year', 'application')
        permissions = (
            ("list_storedocumentnumber", "Can list store document number"),
            ("detail_storedocumentnumber", "Can detail store document number"),
        )

    def __str__(self):
        return '{} - {} - {}: {} - {}'.format(
            self.store_document.name, self.application, self.year,
            self.last_number, self.last_date,
        )


@python_2_unicode_compatible
class StoreCause(DateModel, UserModel, StatusModel):
    objects = StoreCauseManager()
    code = models.CharField(
        primary_key=True,
        max_length=15,
        verbose_name=_('code'),
    )
    store = models.ForeignKey(
        Store,
        verbose_name=_('store'),
    )
    store_document = models.ForeignKey(
        StoreDocument,
        null=True, blank=True,
        verbose_name=_('store document'),
    )
    name = models.CharField(
        unique=True,
        max_length=255,
        db_index=True,
        verbose_name=_('name'),
    )
    description = models.TextField(
        blank=True,
        verbose_name=_('description'),
    )
    category = models.CharField(
        max_length=3,
        choices=settings.STORE_ORGANIZATIONTYPE_DEF,
        db_index=True,
        verbose_name=_('category'),
    )
    move = models.CharField(
        max_length=3,
        choices=settings.STORE_MOVE_TYPE,
        db_index=True,
        verbose_name=_('move'),
    )
    ddt_required = models.BooleanField(
        db_index=True, default=False,
        verbose_name=_('ddt required'),
    )
    invoice_required = models.BooleanField(
        db_index=True, default=False,
        verbose_name=_('invoice required'),
    )
    stock = models.IntegerField(
        choices=settings.STORE_STOCK_BUSY_TYPE,
        db_index=True,
        verbose_name=_('stock'),
    )
    busy = models.IntegerField(
        choices=settings.STORE_STOCK_BUSY_TYPE,
        db_index=True,
        verbose_name=_('busy'),
    )
    default = models.BooleanField(
        default=False,
        verbose_name=_('default'),
    )

    class Meta:
        verbose_name = _('store cause')
        verbose_name_plural = _('store causes')
        permissions = (
            ("list_storecause", "Can list store cause"),
            ("detail_storecause", "Can detail store cause"),
            ("disable_storecause", "Can disable store cause"),
        )

    def __str__(self):
        return '{} {}'.format(self.code, self.name)

    def save(self, *args, **kwargs):
        if self.default:
            default_storecauses = StoreCause.objects.filter(
                store=self.store, category=self.category
            ).exclude(
                pk=self.pk
            )
            for store_cause in default_storecauses:
                store_cause.default = False
                store_cause.save()
        super(StoreCause, self).save(*args, **kwargs)
